@@ -1,4 +1,4 @@
-# Copyright (C) 2023,2024 Giovanni Fulco
+# Copyright (C) 2023,2024,2025 Giovanni Fulco
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@ from subsonic_connector.song import Song
 from tag_type import TagType
 from retrieved_art import RetrievedArt
 
+from tag_to_entry_context import TagToEntryContext
+
 import connector_provider
 import request_cache
 import subsonic_util
@@ -37,24 +39,18 @@ import constants
 import secrets
 
 from typing import Callable
-
-import cmdtalkplugin
-import datetime
-
-# Func name to method mapper
-dispatcher = cmdtalkplugin.Dispatch()
-# Pipe message handler
-msgproc = cmdtalkplugin.Processor(dispatcher)
+from msgproc_provider import msgproc
 
 
-def __get_cover_art_from_res_album_list(response: Response[AlbumList], random: bool = False) -> RetrievedArt:
+def _get_cover_art_from_res_album_list(response: Response[AlbumList], random: bool = False) -> RetrievedArt:
     if not response.isOk() or len(response.getObj().getAlbums()) == 0:
         return None
     album_list: list[Album] = response.getObj().getAlbums()
-    return __get_cover_art_from_album_list(album_list=album_list, random=random)
+    return _get_cover_art_from_album_list(album_list=album_list, random=random)
 
 
-def __get_cover_art_from_album_list(album_list: list[Album], random: bool = False) -> RetrievedArt:
+def _get_cover_art_from_album_list(album_list: list[Album], random: bool = False) -> RetrievedArt:
+    album_list = _filter_album_without_cover_art(album_list=album_list)
     if not album_list or len(album_list) == 0:
         return None
     album: Album = (secrets.choice(album_list)
@@ -64,73 +60,121 @@ def __get_cover_art_from_album_list(album_list: list[Album], random: bool = Fals
     return RetrievedArt(art_url=album_url) if album_url else None
 
 
-def group_albums_art_retriever() -> RetrievedArt:
-    # try in favorites (pick random)
-    art: str = favourite_albums_art_retriever()
-    # else random album
-    if not art:
-        art = random_albums_art_retriever()
-    return art
+def _filter_album_without_cover_art(album_list: list[Album]) -> list[Album]:
+    return (list(filter(lambda x: x.getCoverArt() is not None and len(x.getCoverArt()) > 0, album_list))
+            if album_list
+            else [])
 
 
-def group_artists_art_retriever() -> RetrievedArt:
-    # try in favorites (pick random)
-    art: RetrievedArt = favourite_artist_art_retriever()
-    # else random album
-    if not art:
-        art = random_albums_art_retriever()
-    return art
+def consume_random_album_for_cover_art(tag_to_entry_context: TagToEntryContext = None) -> Album:
+    if tag_to_entry_context is None:
+        msgproc.log("consume_random_album_for_cover_art cannot return an album because a context is not available")
+        return None
+    random_album_list: list[Album] = tag_to_entry_context.random_album_list
+    msgproc.log(f"Albums available in tagToEntryContext [{len(random_album_list)}] ...")
+    select_album: Album = None
+    while not select_album and len(random_album_list) > 0:
+        select: Album = random_album_list.pop(0)
+        # does the album have a cover art?
+        if select.getCoverArt() is not None:
+            msgproc.log(f"consume_random_album_for_cover_art consumed album [{select.getId()}] [{select.getTitle()}] "
+                        f"by [{subsonic_util.get_album_display_artist(album=select)}]")
+            return select
+        else:
+            msgproc.log(f"Skipping album [{select.getId()}] [{select.getTitle()}] "
+                        f"from [{subsonic_util.get_album_display_artist(album=select)}] (no cover art)")
+    msgproc.log("consume_random_album_for_cover_art cannot not return an album")
+    return None
 
 
-def group_songs_art_retriever() -> RetrievedArt:
-    # try in favorites (pick random)
-    art = __art_for_favourite_song(random=True)
+def group_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    msgproc.log(f"group_albums_art_retriever tag_to_entry_context is set: [{tag_to_entry_context is not None}]")
+    # try in favorites (pick random), if allowed
+    art: RetrievedArt = (favourite_albums_art_retriever()
+                         if config.get_config_param_as_bool(constants.ConfigParam.ALLOW_FAVORITES_FOR_FRONT_PAGE_TAGS)
+                         else None)
+    # else random album from context, if available
+    random_album: Album = consume_random_album_for_cover_art(tag_to_entry_context=tag_to_entry_context)
+    # set RetrievedArt if album is found
+    art = (RetrievedArt(art_url=subsonic_util.build_cover_art_url(item_id=random_album.getCoverArt()))
+           if random_album
+           else None)
     # else random
     if not art:
         art = random_albums_art_retriever()
     return art
 
 
-def newest_albums_art_retriever() -> RetrievedArt:
-    response: Response[AlbumList] = connector_provider.get().getAlbumList(
-        ltype=ListType.BY_YEAR,
-        size=1,
-        fromYear=datetime.datetime.now().year,
-        toYear=0)
-    return __get_cover_art_from_res_album_list(response=response)
+def group_artists_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    # try in favorites (pick random)
+    art: RetrievedArt = (favourite_artist_art_retriever()
+                         if config.get_config_param_as_bool(constants.ConfigParam.ALLOW_FAVORITES_FOR_FRONT_PAGE_TAGS)
+                         else None)
+    # else random album from context, if available
+    random_album: Album = consume_random_album_for_cover_art(tag_to_entry_context=tag_to_entry_context)
+    # set RetrievedArt if album is found
+    art = (RetrievedArt(art_url=subsonic_util.build_cover_art_url(item_id=random_album.getCoverArt()))
+           if random_album
+           else None)
+    if not art:
+        art = random_albums_art_retriever()
+    return art
 
 
-def recently_added_albums_art_retriever() -> RetrievedArt:
+def group_songs_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    # try in favorites (pick random)
+    art: RetrievedArt = (_art_for_favourite_song(random=True)
+                         if config.get_config_param_as_bool(constants.ConfigParam.ALLOW_FAVORITES_FOR_FRONT_PAGE_TAGS)
+                         else None)
+    # else random album from context, if available
+    random_album: Album = consume_random_album_for_cover_art(tag_to_entry_context=tag_to_entry_context)
+    # set RetrievedArt if album is found
+    art = (RetrievedArt(art_url=subsonic_util.build_cover_art_url(item_id=random_album.getCoverArt()))
+           if random_album
+           else None)
+    # else random
+    if not art:
+        art = random_albums_art_retriever()
+    return art
+
+
+def newest_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    response: Response[AlbumList] = request_cache.get_first_newest_album_list()
+    return _get_cover_art_from_res_album_list(response=response, random=False)
+
+
+def recently_added_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
     response: Response[AlbumList] = connector_provider.get().getNewestAlbumList(size=1)
-    return __get_cover_art_from_res_album_list(response=response)
+    return _get_cover_art_from_res_album_list(response=response)
 
 
-def random_albums_art_retriever() -> RetrievedArt:
+def random_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    random_album_from_context: Album = consume_random_album_for_cover_art(tag_to_entry_context=tag_to_entry_context)
+    if random_album_from_context:
+        return RetrievedArt(art_url=subsonic_util.build_cover_art_url(item_id=random_album_from_context.getCoverArt()))
     response: Response[AlbumList] = request_cache.get_random_album_list(size=config.get_items_per_page())
-    return __get_cover_art_from_res_album_list(response=response)
+    return _get_cover_art_from_res_album_list(response=response, random=True)
 
 
-def recently_played_albums_art_retriever() -> RetrievedArt:
+def recently_played_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
     response: Response[AlbumList] = connector_provider.get().getAlbumList(ltype=ListType.RECENT, size=1)
-    return __get_cover_art_from_res_album_list(response=response)
+    return _get_cover_art_from_res_album_list(response=response)
 
 
-def highest_rated_albums_art_retriever() -> RetrievedArt:
+def highest_rated_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
     response: Response[AlbumList] = connector_provider.get().getAlbumList(ltype=ListType.HIGHEST, size=1)
-    return __get_cover_art_from_res_album_list(response=response)
+    return _get_cover_art_from_res_album_list(response=response)
 
 
-def most_played_albums_art_retriever() -> RetrievedArt:
+def most_played_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
     response: Response[AlbumList] = connector_provider.get().getAlbumList(ltype=ListType.FREQUENT, size=1)
-    return __get_cover_art_from_res_album_list(response=response)
+    return _get_cover_art_from_res_album_list(response=response)
 
 
 def get_album_art_uri_for_artist(artist: Artist, force_save: bool = False) -> str:
     # best option, the artist has its coverArt
     artist_cover_art: str = subsonic_util.get_artist_cover_art(artist=artist)
     if artist_cover_art:
-        # msgproc.log(f"get_album_art_uri_for_artist for artist [{artist.getId()}] [{artist.getName()}] ->"
-        #             f"coverArt [{artist_cover_art}], building url ...")
         return subsonic_util.build_cover_art_url(item_id=artist_cover_art, force_save=force_save)
     # maybe look at the albums if available
     album_list: list[Album] = artist.getAlbumList()
@@ -165,17 +209,18 @@ def get_album_art_uri_for_artist(artist: Artist, force_save: bool = False) -> st
 
 
 def get_album_art_uri_for_artist_id(artist_id: str) -> str:
+    verbose: bool = config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING)
     artist_metadata: persistence.ArtistMetadata = persistence.get_artist_metadata(artist_id=artist_id)
     if artist_metadata and artist_metadata.artist_cover_art:
         # found in cache.
-        if config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING):
+        if verbose:
             msgproc.log(f"get_album_art_uri_for_artist_id metadata cache hit for [{artist_id}] -> "
                         f"[{'yes' if artist_metadata else 'no'}]")
         return subsonic_util.build_cover_art_url(item_id=artist_metadata.artist_cover_art)
     # fallback to in-memory cache.
     art_album_id: str = cache_actions.get_album_id_by_artist_id(artist_id=artist_id)
     if art_album_id:
-        if config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING):
+        if verbose:
             msgproc.log(f"get_album_art_uri_for_artist_id cache hit for [{artist_id}] -> [{art_album_id}]")
         art_album: Album = subsonic_util.try_get_album(art_album_id)
         if not art_album:
@@ -184,20 +229,20 @@ def get_album_art_uri_for_artist_id(artist_id: str) -> str:
             cache_actions.delete_album_by_artist_id(artist_id)
             persistence.delete_album_metadata(art_album_id)
         elif art_album.getCoverArt():
-            art_album_cover_art_uri: str = (subsonic_util.build_cover_art_url(art_album.getCoverArt())
+            art_album_cover_art_uri: str = (subsonic_util.build_cover_art_url(item_id=art_album.getCoverArt())
                                             if art_album
                                             else None)
             return art_album_cover_art_uri
-    if config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING):
+    if verbose:
         msgproc.log(f"get_album_art_uri_for_artist_id loading artist by id [{artist_id}] ...")
     # load artist
     artist: Artist = subsonic_util.try_get_artist(artist_id=artist_id)
     artist_cover_art: str = subsonic_util.get_artist_cover_art(artist=artist) if artist else None
     if artist_cover_art:
         # use this!
-        if config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING):
+        if verbose:
             msgproc.log(f"get_album_art_uri_for_artist_id using artist coverArt for artist_id [{artist_id}]")
-        return subsonic_util.build_cover_art_url(artist_cover_art)
+        return subsonic_util.build_cover_art_url(item_id=artist_cover_art)
     album_list: list[Album] = artist.getAlbumList() if artist else []
     select_album_list: list[Album] = album_list
     # try to avoid "appearances" first
@@ -205,7 +250,7 @@ def get_album_art_uri_for_artist_id(artist_id: str) -> str:
         artist_id=artist_id,
         album_list=album_list,
         opposite=True)
-    if config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING):
+    if verbose:
         msgproc.log(f"get_album_art_uri_for_artist_id select_album_list for [{artist_id}] "
                     f"[{artist.getName() if artist else None}] "
                     f"full length [{len(album_list) if album_list else 0}] "
@@ -217,7 +262,7 @@ def get_album_art_uri_for_artist_id(artist_id: str) -> str:
     if select_album:
         cache_actions.on_album_for_artist_id(artist_id=artist_id, album=select_album)
         cache_actions.on_album(album=select_album)
-    album_art_uri: str = (subsonic_util.build_cover_art_url(select_album.getCoverArt())
+    album_art_uri: str = (subsonic_util.build_cover_art_url(item_id=select_album.getCoverArt())
                           if select_album
                           else None)
     return album_art_uri
@@ -225,7 +270,7 @@ def get_album_art_uri_for_artist_id(artist_id: str) -> str:
 
 def get_artist_art_url_using_albums_by_artist_id(artist_id: str) -> str:
     artist: Artist = subsonic_util.try_get_artist(artist_id)
-    msgproc.log(f"artist_entry_for_album loaded artist for id [{artist_id}] -> "
+    msgproc.log(f"get_artist_art_url_using_albums_by_artist_id found artist for id [{artist_id}] -> "
                 f"[{'yes' if artist else 'no'}]")
     return get_artist_art_url_using_albums_by_artist(artist=artist) if artist else None
 
@@ -239,7 +284,7 @@ def get_artist_art_url_using_albums_by_artist(artist: Artist) -> str:
         artist_id=artist_id,
         album_list=album_list,
         opposite=True)
-    msgproc.log(f"get_artist_art_url_using_albums_by_artist_id select_album_list for [{artist_id}] "
+    msgproc.log(f"get_artist_art_url_using_albums_by_artist select_album_list for [{artist_id}] "
                 f"[{artist.getName() if artist else None}] "
                 f"full length [{len(album_list) if album_list else 0}] "
                 f"as main artist only [{len(as_main_artist_album_list)}]")
@@ -251,7 +296,7 @@ def get_artist_art_url_using_albums_by_artist(artist: Artist) -> str:
     msgproc.log(f"artist_entry_for_album selected album: [{select_album.getId() if select_album else None}]")
     loaded_album: Album = subsonic_util.try_get_album(album_id=select_album.getId()) if select_album else None
     select_cover: str = loaded_album.getCoverArt() if loaded_album else None
-    return subsonic_util.build_cover_art_url(select_cover) if select_cover else None
+    return subsonic_util.build_cover_art_url(item_id=select_cover) if select_cover else None
 
 
 def get_first_album_with_cover_art_from_album_list(album_list: list[Album]) -> str:
@@ -271,50 +316,76 @@ def get_first_cover_art_from_album_list(album_list: list[Album]) -> str:
     return album.getCoverArt() if album else None
 
 
-def favourite_albums_art_retriever() -> RetrievedArt:
-    fav: RetrievedArt = __favourite_albums_art_retriever()
+def favourite_albums_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    fav: RetrievedArt = _favourite_albums_art_retriever()
     return fav if fav else random_albums_art_retriever()
 
 
-def __favourite_albums_art_retriever() -> RetrievedArt:
+def _favourite_albums_art_retriever() -> RetrievedArt:
     response: Response[Starred] = request_cache.get_starred()
     if not response.isOk():
         return None
     album_list: list[Album] = response.getObj().getAlbums()
-    return __get_cover_art_from_album_list(album_list=album_list, random=True)
+    return _get_cover_art_from_album_list(album_list=album_list, random=True)
 
 
-def favourite_artist_art_retriever() -> RetrievedArt:
-    fav: RetrievedArt = __favourite_artist_art_retriever()
+def favourite_artist_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    fav: RetrievedArt = _favourite_artist_art_retriever()
     msgproc.log(f"favourite_artist_art_retriever fav found: [{'yes' if fav else 'no'}]")
     return fav if fav is not None else random_albums_art_retriever()
 
 
-def __favourite_artist_art_retriever() -> RetrievedArt:
+def _favourite_artist_art_retriever() -> RetrievedArt:
+    verbose: bool = config.get_config_param_as_bool(constants.ConfigParam.VERBOSE_LOGGING)
     response: Response[Starred] = request_cache.get_starred()
     if not response.isOk():
-        msgproc.log("__favourite_artist_art_retriever no starred artists")
+        msgproc.log("_favourite_artist_art_retriever no starred artists")
         return None
     artist_list: list[Artist] = response.getObj().getArtists()
-    select_artist: Artist = secrets.choice(artist_list) if artist_list and len(artist_list) > 0 else None
-    msgproc.log("__favourite_artist_art_retriever selected artist "
-                f"[{select_artist.getId() if select_artist else None}]")
+    with_id: list[Artist] = list(filter(lambda a: a.getId() is not None, artist_list if artist_list else []))
+    select_artist: Artist = secrets.choice(with_id) if with_id and len(with_id) > 0 else None
+    msgproc.log(f"_favourite_artist_art_retriever select_artist is set: {'yes' if select_artist else 'no'}")
+    msgproc.log("_favourite_artist_art_retriever selected artist "
+                f"[{select_artist.getId() if select_artist else None}] "
+                f"[{select_artist.getName() if select_artist else None}]")
     if not select_artist:
         return None
-    art_url: str = get_artist_art_url_using_albums_by_artist_id(artist_id=select_artist.getId())
+    artist_cover_art: str = subsonic_util.get_artist_cover_art(artist=select_artist)
+    art_url: str = None
+    if verbose:
+        msgproc.log("_favourite_artist_art_retriever got cover art from artist "
+                    f"[{select_artist.getId()}] [{select_artist.getName()}]: "
+                    f"[{'yes' if artist_cover_art else 'no'}] "
+                    f"-> CoverArt: [{artist_cover_art}]")
+    if artist_cover_art:
+        msgproc.log(f"_favourite_artist_art_retriever [{select_artist.getId()}] [{select_artist.getName()}] "
+                    f"using artist information (best option)")
+        art_url = subsonic_util.build_cover_art_url(item_id=artist_cover_art)
+        return RetrievedArt(art_url=art_url)
+    # look in cache first
+    select_artist_metadata: persistence.ArtistMetadata = persistence.get_artist_metadata(artist_id=select_artist.getId())
+    if select_artist_metadata and select_artist_metadata.artist_cover_art:
+        msgproc.log(f"_favourite_artist_art_retriever [{select_artist.getId()}] [{select_artist.getName()}] "
+                    f"using artist metadata (second best option)")
+        art_url = subsonic_util.build_cover_art_url(item_id=select_artist_metadata.artist_cover_art)
+        return RetrievedArt(art_url=art_url)
+    # last chance, load artist, get albums, then select one
+    msgproc.log(f"_favourite_artist_art_retriever [{select_artist.getId()}] [{select_artist.getName()}] "
+                f"loading artist, worst option")
+    art_url = get_artist_art_url_using_albums_by_artist_id(artist_id=select_artist.getId())
     return RetrievedArt(art_url=art_url)
 
 
-def favourite_song_retriever() -> RetrievedArt:
-    fav: RetrievedArt = __favourite_song_retriever()
+def favourite_song_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
+    fav: RetrievedArt = _favourite_song_retriever()
     return fav if fav else random_albums_art_retriever()
 
 
-def __favourite_song_retriever() -> RetrievedArt:
-    return __art_for_favourite_song(random=True)
+def _favourite_song_retriever() -> RetrievedArt:
+    return _art_for_favourite_song(random=True)
 
 
-def __art_for_favourite_song(random: bool = False) -> RetrievedArt:
+def _art_for_favourite_song(random: bool = False) -> RetrievedArt:
     response: Response[Starred] = request_cache.get_starred()
     if not response.isOk():
         return None
@@ -328,7 +399,7 @@ def __art_for_favourite_song(random: bool = False) -> RetrievedArt:
     return RetrievedArt(art_url=art_url)
 
 
-def playlists_art_retriever() -> RetrievedArt:
+def playlists_art_retriever(tag_to_entry_context: TagToEntryContext = None) -> RetrievedArt:
     response: Response[Playlists] = request_cache.get_playlists()
     if not response.isOk():
         msgproc.log("playlists_art_retriever - cannot get playlists")
@@ -341,17 +412,19 @@ def playlists_art_retriever() -> RetrievedArt:
     if not select:
         msgproc.log("playlists_art_retriever - no playlist selected")
         return None
-    art_url: str = (subsonic_util.build_cover_art_url(select.getCoverArt())
+    art_url: str = (subsonic_util.build_cover_art_url(item_id=select.getCoverArt())
                     if select and select.getCoverArt()
                     else None)
     return RetrievedArt(art_url=art_url)
 
 
-def execute_art_retriever(tag: TagType) -> RetrievedArt:
+def execute_art_retriever(
+        tag: TagType,
+        context: TagToEntryContext = None) -> RetrievedArt:
     tagname: str = tag.getTagName()
     if tagname in __tag_art_retriever_dict:
         try:
-            return __tag_art_retriever_dict[tagname]()
+            return __tag_art_retriever_dict[tagname](context)
         except Exception as ex:
             msgproc.log(f"Cannot retrieve art for tag [{tagname}] [{type(ex)}] [{ex}]")
 
@@ -362,19 +435,23 @@ __tag_art_retriever_dict: dict[str, Callable[[], RetrievedArt]] = {
     TagType.SONGS.getTagName(): group_songs_art_retriever,
     TagType.RECENTLY_ADDED_ALBUMS.getTagName(): recently_added_albums_art_retriever,
     TagType.NEWEST_ALBUMS.getTagName(): newest_albums_art_retriever,
+    TagType.OLDEST_ALBUMS.getTagName(): random_albums_art_retriever,
     TagType.RECENTLY_PLAYED_ALBUMS.getTagName(): recently_played_albums_art_retriever,
     TagType.HIGHEST_RATED_ALBUMS.getTagName(): highest_rated_albums_art_retriever,
-    TagType.FAVOURITE_ALBUMS.getTagName(): favourite_albums_art_retriever,
+    TagType.FAVORITE_ALBUMS.getTagName(): favourite_albums_art_retriever,
     TagType.MOST_PLAYED_ALBUMS.getTagName(): most_played_albums_art_retriever,
     TagType.RANDOM.getTagName(): random_albums_art_retriever,
+    TagType.ALBUMS_WITHOUT_MUSICBRAINZ.getTagName(): random_albums_art_retriever,
+    TagType.ALBUMS_WITHOUT_COVER.getTagName(): random_albums_art_retriever,
+    TagType.ALBUMS_WITHOUT_GENRE.getTagName(): random_albums_art_retriever,
     TagType.RANDOM_SONGS.getTagName(): random_albums_art_retriever,
     TagType.RANDOM_SONGS_LIST.getTagName(): random_albums_art_retriever,
     TagType.GENRES.getTagName(): random_albums_art_retriever,
     TagType.ALL_ARTISTS.getTagName(): random_albums_art_retriever,
     TagType.ALL_ARTISTS_UNSORTED.getTagName(): random_albums_art_retriever,
     TagType.ALL_ARTISTS_INDEXED.getTagName(): random_albums_art_retriever,
-    TagType.FAVOURITE_ARTISTS.getTagName(): favourite_artist_art_retriever,
+    TagType.FAVORITE_ARTISTS.getTagName(): favourite_artist_art_retriever,
     TagType.PLAYLISTS.getTagName(): playlists_art_retriever,
-    TagType.FAVOURITE_SONGS.getTagName(): favourite_song_retriever,
-    TagType.FAVOURITE_SONGS_LIST.getTagName(): favourite_song_retriever
+    TagType.FAVORITE_SONGS.getTagName(): favourite_song_retriever,
+    TagType.FAVORITE_SONGS_LIST.getTagName(): favourite_song_retriever
 }
